#!/usr/bin/python2
# -*- coding: utf-8 -*-

# ==============================================================================
#      Frank Matranga's Third-party Regis High School Python Module
# ==============================================================================

import sys
import os
from scraper import Scraper
from cli import CLI
from viewer import Viewer
import json
from pymongo import MongoClient
import requests
from lxml import html

PATH = "./secrets.json"

IP = "localhost" if len(sys.argv) < 3 else sys.argv[2]
PORT = "27017" if len(sys.argv) < 4 else sys.argv[3]
DB_NAME = "regis" if len(sys.argv) < 5 else sys.argv[4]

class TRP:
    def __init__(self, path=PATH, ip=IP, port=PORT, db_name=DB_NAME):
        self.path = path
        self.ip = ip
        self.port = port
        self.db_name = db_name

        print ip
        print " --- Initalizing TRP Module ---\n"
        #print "Arguments: "+str(sys.argv[1::])
        self.secrets = self.get_secrets()
        self.username = self.secrets['regis_username']
        self.password = self.secrets['regis_password']
        print "Found info for Regis student '"+self.username+"'"
        self.connect_to_db()
        self.session = self.get_session()
        self.init_mods()
        print "\n --- READY --- \n"

    def get_secrets(self):
        if len(sys.argv) > 1 and os.path.exists(sys.argv[1]):
            self.path = sys.argv[1]

        if os.path.isdir(self.path):
            if self.path[-1] != "/":
                self.path += "/"
            self.path += "secrets.json"


        try:
            secrets = json.loads(open(self.path).read())
        except (ValueError, IOError):
            print "'"+self.path+"' is not a valid secrets.json"
            self.exit()

        print "Using path '"+self.path+"' for secrets.json"
        return secrets

    def connect_to_db(self):
        try:
            print "Attempting to connect to mongodb://"+self.ip+":"+self.port+"/"+self.db_name+"..."
            self.client = MongoClient('mongodb://'+self.ip+':'+self.port+'/')
            self.db = self.client[DB_NAME]
            self.db.authenticate('ontrac', 'ontrac')
            self.db.students.count()
            print "Done."
        except Exception: # nasty I know
            self.client = None
            print "Failed to connect to Database."
            self.exit()

    def get_session(self):
        print "Attempting logins..."
        url = "https://moodle.regis.org/login/index.php"
        values = {'username': self.username, 'password': self.password}
        session = requests.Session()
        r = session.post(url, data=values)
        parsed_body = html.fromstring(r.text)
        title = parsed_body.xpath('//title/text()')[0]

        # Check whether login was successful or not
        if not "My home" in title:
            print "Failed to login to Moodle, check your credentials."
            self.exit()
        print "Logged into Moodle."

        url = "https://intranet.regis.org/login/submit.cfm"
        values = {'username': self.username, 'password': self.password, 'loginsubmit': ''}
        r = session.post(url, data=values)
        parsed_body = html.fromstring(r.text)
        try:
            title = parsed_body.xpath('//title/text()')[0]
            if not "Intranet" in title:
                print "Failed to login to the Intranet, check your credentials."
                quit()
        except:
            print "Failed to login to the Intranet, check your credentials."
            quit()

        print "Logged in to the Intranet."
        print "Done."
        return session

    def init_mods(self):
        print "Initialzing modules..."
        self.viewer = Viewer(self.db)
        self.scraper = Scraper(self.db, self.session)
        self.cli = CLI(self.db)
        print "Done."

    def exit(self):
        try: # this might be run before it is assigned
            self.client.close()
            print "Closed DB connection."
        except:
            pass

        print "Shutting down..."
        quit()
        sys.exit()

def main():
    t = TRP()
    t.exit()

if __name__ == "__main__":
    main()



from setuptools import setup, find_packages
# To use a consistent encoding
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the relevant file
with open(path.join(here, 'DESCRIPTION.rst'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='trp',
    version='0.1.5a1',

    description='Third-party Regis Python Module',
    long_description=long_description,

    url='https://github.com/Apexal/trp',

    author='Frank Matranga',
    author_email='thefrankmatranga@gmail.com',

    license='MIT',

    install_requires=['pymongo', 'requests', 'lxml'],

    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2',
    ],

    keywords='regis moodle scraper education school academics',

    packages=find_packages(),

    entry_points={
        'console_scripts': [
            'trp=trp:main',
        ],
    }
)



#!/usr/bin/python2
# -*- coding: utf-8 -*-
import os
import requests, sys
from lxml import html
from time import sleep
import json
import datetime

class Scraper:
    def __init__(self, db, session):
        print "Initialized Moodle Scraper"
        self.session = session
        self.db = db

    def loop(self, from_id, to_id, category):
        for i in range(from_id, to_id + 1):
            self.extract(i, category)

    def extract(self, ID, category):

        base_url = "http://moodle.regis.org/user/profile.php?id="
        if category is "course":
            base_url = "http://moodle.regis.org/course/view.php?id="
        # Get the page
        r = self.session.get(base_url + str(ID))  # The url is created by appending the current ID to the base url
        # Parse the html returned so we can find the title
        #print r.text
        parsed_body = html.fromstring(r.text)
        # Get the page title
        title = parsed_body.xpath('//title/text()')
        # Check if page is useful
        if len(title) == 0:
            #print "Bad title"
            return

        if "Test" in title:
            #print "Skipped test entry"
            return

        if ("Error" in title[0].strip()) or ("Notice" in title[0].strip()):
            #print "Error or Notice skipped"
            return

        # COURSE
        title = parsed_body.xpath('//title/text()')[0]
        parts = title.split(": ")
        if parts[0] == "Course":
            name = parts[1]
            ps = name.split(" ")

            teacher = parts[2] if len(parts) > 2 else "Unknown"

            if "Advisement " in name:
                out = {
                    "tID": teacher,
                    "title": name.replace("Advisement ", ""),
                    "mID": ID
                }
                print str(ID)+": Advisement " + out['title'] + " " + str(self.db.advisements.insert_one(out).inserted_id)

            else:
                grade = 13
                for pa in ps:
                    for index, g in enumerate(["I", "II", "III", "IV"]):
                        if g == pa:
                            grade = 9 + index
                    try:
                        grade = int(pa)
                    except ValueError:
                        pass
                courseType = "class"
                if "Club" in name or "Society" in name:
                    courseType = "club"

                if "REACH" in name or "Reach" in name:
                    courseType = "reach"

                out = {
                    "full": title,
                    "courseType": courseType,
                    "mID": ID,
                    "title": name,
                    "students": [],
                    "grade": grade
                }
                print str(ID)+": Course " + str(self.db.courses.insert_one(out).inserted_id)
        else:
            # USER
            name_parts = parsed_body.xpath('//title/text()')[0].split(":")[0].split(", ") if len(
                parsed_body.xpath('//title/text()')) > 0 else ['Unknown']
            department = parsed_body.xpath('//dl/dd[1]/text()')
            if len(department) == 0:
                return
            else:
                department = department[0]
            class_as = parsed_body.xpath('//dl/dd[2]/a')

            classes = []
            for a in class_as:
                classes.append(int(a.get("href").split("id=")[1]))

            f = department[0]
            try:
                int(f)
                userType = "student"
            except ValueError:
                userType = "teacher"

            if department.startswith("REACH"):
                userType = "other"

            picsrc = "/images/person-placeholder.jpg"

            for img in parsed_body.xpath('//img[@class=\'userpicture\']'):
                picsrc = img.get("src")

            collect = self.db.courses
            courses = []

            intranet = self.session.get("http://intranet.regis.org/infocenter/default.cfm?FuseAction=basicsearch&searchtype=namewildcard&criteria="+name_parts[0]+"%2C+"+name_parts[1]+"&[whatever]=Search")
            intrabody = html.fromstring(intranet.text)

            schedule = "<h2>Not Found</h2>"

            if userType == "student":
                style="float:left;width:200px;"
            else:
                style="float:left; width:200px;"

            if userType == "student":
                if classes:
                    adv = self.db.advisements.find_one({"mID": classes[0]})
                    if adv:
                        department = adv['title']

            search_results = intrabody.xpath('//div[@style="'+style+'"]/span[1]/text()')
            #print search_results
            if len(search_results) == 0:
                print str(ID)+": Found on Moodle but not the Intranet. Disregarding."
                return # should people found only on Moodle be included?

            #print "Intranet found "+str(len(search_results))+" search results for", name_parts
            for result in search_results:
                index = search_results.index(result)
                name_p = result.encode('utf-8').split(", ")
                intranet_dep = name_p[1].split(" ")[1].replace("(", "").replace(")", "")

                first_name = name_p[1].split(" ")[0]
                last_name = name_p[0]
                #print "Attempting to match "+ str([name_parts[0], name_parts[1], department])+" with " + str([last_name, first_name, intranet_dep])
                if [last_name, first_name] == name_parts:
                    if userType == "student":
                        if intranet_dep == department:
                            #print "Yup!"
                            break
                    else:
                        #print "Yup!"
                        break
                #print "Nope"

            email = intrabody.xpath('//div[@style="'+style+'"]/span[2]/a/text()')[index]
            #print email
            username = str(email).replace("@regis.org", "").lower()

            pic_elm = intrabody.xpath('//div[@style="'+style+'"]/a')[index]
            code = pic_elm.get("href").split("/")[-1].replace(".jpg", "")

            if userType == "student":
                style="float:left;width:200px;"
                scheduleurl = "http://intranet.regis.org/infocenter?StudentCode="+code
            else:
                style="float:left; width:200px;"
                scheduleurl = "http://intranet.regis.org/infocenter/default.cfm?StaffCode="+code

            htmlschedulereq = self.session.get(scheduleurl)
            htmls = html.fromstring(htmlschedulereq.text)
            if len(htmls.xpath('//div[@id="main"]/table[4]')) > 0:
                schedule = html.tostring(htmls.xpath('//div[@id="main"]/table[4]')[0])

            try:
                if userType == "student":
                    out = {
                        "mID": ID,
                        "firstName": name_parts[1],
                        "lastName": name_parts[0],
                        "username": username,
                        "code": code,
                        "mpicture": picsrc,
                        "ipicture": pic_elm.get("href"),
                        "schedule": schedule,
                        "email": username + "@regis.org",
                        "advisement": department,
                        "courses": courses,
                        "sclasses": classes,
                    }
                    newID = self.db.students.insert_one(out).inserted_id
                    print str(ID)+": Student " + username + " in Advisement " + department + " with Student ID "+code+" in "+str(len(classes))+" courses"
                    if classes:
                        self.db.advisements.update_one({"mID": classes[0]}, {"$push": {"students": newID} } )
                        for c in classes: # C IS A MOODLE ID
                            course = collect.find_one({"mID": c})
                            if course:
                                cID = course['_id']
                                collect.update_one({"mID": c}, {"$push": {"students": newID}})
                                self.db.students.update_one({"_id": newID}, {"$push": {"courses": cID}})

                else:
                    print str(ID)+": Staff Member " + username + " of the " + department + " Department with Staff ID "+code+" in "+str(len(classes))+" courses"
                    out = {
                        "userType": userType,
                        "mID": ID,
                        "image": picsrc,
                        "code": code,
                        "ipicture": pic_elm.get("href"),
                        "department": department,
                        "firstName": name_parts[1],
                        "lastName": name_parts[0],
                        "schedule": schedule,
                        "username": username,
                        "email": email,
                        "sclasses": classes,
                        "courses": courses
                    }
                    newID = self.db.teachers.insert_one(out).inserted_id
                    #print "Teacher " + str(ID) + ": " + str(newID)
                    for c in classes:
                        course = collect.find_one({"mID": c})
                        if course:
                            if name_parts[0] in course["full"]:
                                #print "COURSE:", course['title']
                                collect.update_one({
                                  '_id': course['_id']
                                },{
                                  '$set': {
                                    'teacher': newID
                                  }
                                }, upsert=False)
                            self.db.teachers.update_one({"_id": newID}, {"$push": {"courses": course['_id']}})
                        adv = self.db.advisements.find_one({"mID": c})
                        if adv:
                            self.db.advisements.update_one({
                              'mID': c
                            },{
                              '$set': {
                                'teacher': newID
                              }
                            })
                            #print "Set Teacher "+str(ID)+" as Advisor of "+adv['title']

            #raw_input("Continue?")
            except Exception as e:
                print e



import scraper

COMMANDS = ['help', 'scrape', 'exit']

running = True

class CLI:
    def __init__(self, db):
        print "Initialized CLI"

    def is_valid_command(self, com):
        com = com.split(" ")
        if not com[0] in COMMANDS:
            return False
        else:
            return True

    def run_command(self, input, session):
        split = input.split(" ")
        c = split[0]
        args = split[1::]
        #print c, args

        globals()[c](args, session)

    def run(self):
        print "[COMMAND LINE]"
        while running:
            com = raw_input("> ")
            if self.is_valid_command(com):
                self.run_command(com, None)
            else:
                print "Command not found. Type 'help' for a list of commands."


################## COMMANDS ##################
def exit(args, session):
    global running
    running = False

def help(args, session):
    print "\nCommand list:"
    for com in COMMANDS:
        print "* "+com
    print ""
